@@ -30,6 +30,7 @@
 * A simple application using libqalculate need only create a calculator object, perhaps load definitions (functions, variables, units, etc.), and calculate (and output) an expression as follows:
 * \code 
 * new Calculator();
+* CALCULATOR->loadExchangeRates();
 * CALCULATOR->loadGlobalDefinitions();
 * CALCULATOR->loadLocalDefinitions();
 * cout << CALCULATOR->calculateAndPrint("1 + 1", 2000) << endl;\endcode
@@ -154,6 +155,7 @@ typedef enum {
 ///Message categories
 #define MESSAGE_CATEGORY_NONE		0
 #define MESSAGE_CATEGORY_PARSING	1
+#define MESSAGE_CATEGORY_WIDE_INTERVAL	10
 
 
 /// A message with information to the user. Primarily used for errors and warnings.
@@ -350,6 +352,7 @@ class Calculator {
 	/** @name Constructor */
 	//@{
 	Calculator();
+	Calculator(bool ignore_locale);
 	virtual ~Calculator();
 	//@}
 
@@ -368,7 +371,7 @@ class Calculator {
 	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
 	* @returns true if the calculation was successfully started (and finished if msecs > 0).
 	*/
-	bool calculate(MathStructure *mstruct, string str, int msecs, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
+	bool calculate(MathStructure *mstruct, string str, int msecs, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
 	/** Calculates an expression. The expression should be unlocalized first with unlocalizeExpression().
 	*
 	* @param str Expression.
@@ -378,7 +381,7 @@ class Calculator {
 	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
 	* @returns The result of the calculation.
 	*/
-	MathStructure calculate(string str, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
+	MathStructure calculate(string str, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
 	/** Calculates a parsed value.
 	* This function starts the calculation in a separate thread and will return when the calculation has started unless a maximum time has been specified.
 	* The calculation can then be stopped with abort().
@@ -389,7 +392,7 @@ class Calculator {
 	* @param to_str "to" expression for conversion.
 	* @returns The result of the calculation.
 	*/
-	bool calculate(MathStructure *mstruct, int msecs, const EvaluationOptions &eo = default_evaluation_options, string to_str = "");
+	bool calculate(MathStructure *mstruct, int msecs, const EvaluationOptions &eo = default_user_evaluation_options, string to_str = "");
 	/** Calculates a parsed value.
 	*
 	* @param mstruct Parsed value to evaluate.
@@ -397,7 +400,7 @@ class Calculator {
 	* @param to_str "to" expression for conversion.
 	* @returns The result of the calculation.
 	*/
-	MathStructure calculate(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options, string to_str = "");
+	MathStructure calculate(const MathStructure &mstruct, const EvaluationOptions &eo = default_user_evaluation_options, string to_str = "");
 	/** Calculates an expression.and outputs the result to a text string. The expression should be unlocalized first with unlocalizeExpression().
 	* 
 	* Unlike other functions for expression evaluation this function handles ending "to"-commmands, in addition to unit conversion, such "to hexadecimal" or to "fractions", similar to the qalc application.
@@ -410,7 +413,7 @@ class Calculator {
 	* @returns The result of the calculation.
 	* \since 2.6.0
 	*/
-	string calculateAndPrint(string str, int msecs = 10000, const EvaluationOptions &eo = default_evaluation_options, const PrintOptions &po = default_print_options);
+	string calculateAndPrint(string str, int msecs = 10000, const EvaluationOptions &eo = default_user_evaluation_options, const PrintOptions &po = default_print_options);
 	int testCondition(string expression);
 	//@}
 
@@ -480,7 +483,7 @@ class Calculator {
 	* @param eo Options for the evaluation and parsing of the expression.
 	* @returns true if the calculation was successfully started (and finished if msecs > 0).
 	*/
-	bool calculateRPNRegister(size_t index, int msecs, const EvaluationOptions &eo = default_evaluation_options);
+	bool calculateRPNRegister(size_t index, int msecs, const EvaluationOptions &eo = default_user_evaluation_options);
 	/** Applies a mathematical operation to the first and second value on the RPN stack. The the second value is changed with input from the first value.
 	* For example, with OPERATION_SUBTRACT the first value is subtracted from the second. The first value on the stack is removed.
 	* If not enough registers is available, then zeros are added.
@@ -493,7 +496,7 @@ class Calculator {
 	* @param[out] parsed_struct NULL or a math structure to fill with the unevaluated result.
 	* @returns true if the calculation was successfully started (and finished if msecs > 0).
 	*/
-	bool calculateRPN(MathOperation op, int msecs, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL);
+	bool calculateRPN(MathOperation op, int msecs, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL);
 	/** Applies a mathematical operation to the first value on the RPN stack. The value is set as the first argument of the function.
 	* If no register is available, then zero is added.
 	* This function starts the calculation in a separate thread and will return when the calculation has started unless a maximum time has been specified.
@@ -505,7 +508,7 @@ class Calculator {
 	* @param[out] parsed_struct NULL or a math structure to fill with the unevaluated result.
 	* @returns true if the calculation was successfully started (and finished if msecs > 0).
 	*/
-	bool calculateRPN(MathFunction *f, int msecs, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL);
+	bool calculateRPN(MathFunction *f, int msecs, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL);
 	/** Applies bitwise not to the first value on the RPN stack.
 	* If no register is available, then zero is added.
 	* This function starts the calculation in a separate thread and will return when the calculation has started unless a maximum time has been specified.
@@ -516,7 +519,7 @@ class Calculator {
 	* @param[out] parsed_struct NULL or a math structure to fill with the unevaluated result.
 	* @returns true if the calculation was successfully started (and finished if msecs > 0).
 	*/
-	bool calculateRPNBitwiseNot(int msecs, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL);
+	bool calculateRPNBitwiseNot(int msecs, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL);
 	/** Applies logical not to the first value on the RPN stack.
 	* If no register is available, then zero is added.
 	* This function starts the calculation in a separate thread and will return when the calculation has started unless a maximum time has been specified.
@@ -527,7 +530,7 @@ class Calculator {
 	* @param[out] parsed_struct NULL or a math structure to fill with the unevaluated result.
 	* @returns true if the calculation was successfully started (and finished if msecs > 0).
 	*/
-	bool calculateRPNLogicalNot(int msecs, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL);
+	bool calculateRPNLogicalNot(int msecs, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL);
 	/** Applies a mathematical operation to the first and second value on the RPN stack. The the second value is changed with input from the first value.
 	* For example, with OPERATION_SUBTRACT the first value is subtracted from the second. The first value on the stack is removed.
 	* If not enough registers is available, then zeros are added.
@@ -537,7 +540,7 @@ class Calculator {
 	* @param[out] parsed_struct NULL or a math structure to fill with the unevaluated result.
 	* @returns The first value on the stack.
 	*/
-	MathStructure *calculateRPN(MathOperation op, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL);
+	MathStructure *calculateRPN(MathOperation op, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL);
 	/** Applies a mathematical operation to the first value on the RPN stack. The value is set as the first argument of the function.
 	* If no register is available, then zero is added.
 	*
@@ -546,7 +549,7 @@ class Calculator {
 	* @param[out] parsed_struct NULL or a math structure to fill with the unevaluated result.
 	* @returns The first value on the stack.
 	*/
-	MathStructure *calculateRPN(MathFunction *f, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL);
+	MathStructure *calculateRPN(MathFunction *f, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL);
 	/** Applies bitwise not to the first value on the RPN stack.
 	* If no register is available, then zero is added.
 	*
@@ -554,7 +557,7 @@ class Calculator {
 	* @param[out] parsed_struct NULL or a math structure to fill with the unevaluated result.
 	* @returns The first value on the stack.
 	*/
-	MathStructure *calculateRPNBitwiseNot(const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL);
+	MathStructure *calculateRPNBitwiseNot(const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL);
 	/** Applies logical not to the first value on the RPN stack.
 	* If no register is available, then zero is added.
 	*
@@ -562,7 +565,7 @@ class Calculator {
 	* @param[out] parsed_struct NULL or a math structure to fill with the unevaluated result.
 	* @returns The first value on the stack.
 	*/
-	MathStructure *calculateRPNLogicalNot(const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL);
+	MathStructure *calculateRPNLogicalNot(const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL);
 	/** Evaluates a value and adds the result first on the RPN stack.
 	* This function starts the calculation in a separate thread and will return when the calculation has started unless a maximum time has been specified.
 	* The calculation can then be stopped with abort().
@@ -572,7 +575,7 @@ class Calculator {
 	* @param eo Options for the evaluation of the expression.
 	* @returns true if the calculation was successfully started (and finished if msecs > 0).
 	*/
-	bool RPNStackEnter(MathStructure *mstruct, int msecs, const EvaluationOptions &eo = default_evaluation_options);
+	bool RPNStackEnter(MathStructure *mstruct, int msecs, const EvaluationOptions &eo = default_user_evaluation_options);
 	/** Calculates an expression and adds the result first on the RPN stack. The expression should be unlocalized first with unlocalizeExpression().
 	* This function starts the calculation in a separate thread and will return when the calculation has started unless a maximum time has been specified.
 	* The calculation can then be stopped with abort().
@@ -585,13 +588,13 @@ class Calculator {
 	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
 	* @returns true if the calculation was successfully started (and finished if msecs > 0).
 	*/
-	bool RPNStackEnter(string str, int msecs, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
+	bool RPNStackEnter(string str, int msecs, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
 	/** Adds a value first on the RPN stack.
 	*
 	* @param mstruct Value.
 	* @param eval If true, the the mathematical structure will be evaluated first.
 	*/
-	void RPNStackEnter(MathStructure *mstruct, bool eval = false, const EvaluationOptions &eo = default_evaluation_options);
+	void RPNStackEnter(MathStructure *mstruct, bool eval = false, const EvaluationOptions &eo = default_user_evaluation_options);
 	/** Calculates an expression adds the result first on the RPN stack. The expression should be unlocalized first with unlocalizeExpression().
 	*
 	* @param str Expression.
@@ -600,11 +603,11 @@ class Calculator {
 	* @param[out] to_struct NULL or a math structure to fill with unit expression parsed after "to".
 	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
 	*/
-	void RPNStackEnter(string str, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
-	bool setRPNRegister(size_t index, MathStructure *mstruct, int msecs, const EvaluationOptions &eo = default_evaluation_options);
-	bool setRPNRegister(size_t index, string str, int msecs, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
-	void setRPNRegister(size_t index, MathStructure *mstruct, bool eval = false, const EvaluationOptions &eo = default_evaluation_options);
-	void setRPNRegister(size_t index, string str, const EvaluationOptions &eo = default_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
+	void RPNStackEnter(string str, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
+	bool setRPNRegister(size_t index, MathStructure *mstruct, int msecs, const EvaluationOptions &eo = default_user_evaluation_options);
+	bool setRPNRegister(size_t index, string str, int msecs, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
+	void setRPNRegister(size_t index, MathStructure *mstruct, bool eval = false, const EvaluationOptions &eo = default_user_evaluation_options);
+	void setRPNRegister(size_t index, string str, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
 	void deleteRPNRegister(size_t index);
 	MathStructure *getRPNRegister(size_t index = 1) const;
 	size_t RPNStackSize() const;
@@ -664,7 +667,7 @@ class Calculator {
 	* @param[out] units NULL or a math structure to fill with the parsed unit expression(or set to undefined if no units were found).
 	* @returns Converted value.
 	*/
-	MathStructure convert(const MathStructure &mstruct, string composite_, const EvaluationOptions &eo = default_evaluation_options, MathStructure *units = NULL);
+	MathStructure convert(const MathStructure &mstruct, string composite_, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *units = NULL);
 	/** Converts to a unit.
 	* The converted value is evaluated.
 	*
@@ -674,22 +677,22 @@ class Calculator {
 	* @param always_convert ...
 	* @returns Converted value.
 	*/
-	MathStructure convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options, bool always_convert = true, bool convert_to_mixed_units = true);
-	MathStructure convert(const MathStructure &mstruct, KnownVariable *to_var, const EvaluationOptions &eo = default_evaluation_options);
-	MathStructure convert(double value, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options);
-	MathStructure convert(string str, Unit *from_unit, Unit *to_unit, int milliseconds, const EvaluationOptions &eo = default_evaluation_options);
+	MathStructure convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo = default_user_evaluation_options, bool always_convert = true, bool convert_to_mixed_units = true);
+	MathStructure convert(const MathStructure &mstruct, KnownVariable *to_var, const EvaluationOptions &eo = default_user_evaluation_options);
+	MathStructure convert(double value, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_user_evaluation_options);
+	MathStructure convert(string str, Unit *from_unit, Unit *to_unit, int milliseconds, const EvaluationOptions &eo = default_user_evaluation_options);
 	/** Depecated: use convert() */
-	MathStructure convertTimeOut(string str, Unit *from_unit, Unit *to_unit, int milliseconds, const EvaluationOptions &eo = default_evaluation_options);
-	MathStructure convert(string str, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_evaluation_options);	
-	MathStructure convertToBaseUnits(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options);
+	MathStructure convertTimeOut(string str, Unit *from_unit, Unit *to_unit, int milliseconds, const EvaluationOptions &eo = default_user_evaluation_options);
+	MathStructure convert(string str, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_user_evaluation_options);	
+	MathStructure convertToBaseUnits(const MathStructure &mstruct, const EvaluationOptions &eo = default_user_evaluation_options);
 	/** Deprecated: use getOptimalUnit() */
 	Unit *getBestUnit(Unit *u, bool allow_only_div = false, bool convert_to_local_currency = true);
 	Unit *getOptimalUnit(Unit *u, bool allow_only_div = false, bool convert_to_local_currency = true);
 	/** Deprecated: use convertToOptimalUnit() */
-	MathStructure convertToBestUnit(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options, bool convert_to_si_units = true);
-	MathStructure convertToOptimalUnit(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options, bool convert_to_si_units = true);
-	MathStructure convertToCompositeUnit(const MathStructure &mstruct, CompositeUnit *cu, const EvaluationOptions &eo = default_evaluation_options, bool always_convert = true);
-	MathStructure convertToMixedUnits(const MathStructure &mstruct, const EvaluationOptions &eo = default_evaluation_options);
+	MathStructure convertToBestUnit(const MathStructure &mstruct, const EvaluationOptions &eo = default_user_evaluation_options, bool convert_to_si_units = true);
+	MathStructure convertToOptimalUnit(const MathStructure &mstruct, const EvaluationOptions &eo = default_user_evaluation_options, bool convert_to_si_units = true);
+	MathStructure convertToCompositeUnit(const MathStructure &mstruct, CompositeUnit *cu, const EvaluationOptions &eo = default_user_evaluation_options, bool always_convert = true);
+	MathStructure convertToMixedUnits(const MathStructure &mstruct, const EvaluationOptions &eo = default_user_evaluation_options);
 	//@}
 
 	/** Used by the UI to find unit category for a mathematical expression.*/
@@ -830,6 +833,7 @@ class Calculator {
 	Unit* getActiveUnit(string name_);
 	Unit* getCompositeUnit(string internal_name_);
 	Unit* getLocalCurrency();
+	void setLocalCurrency(Unit *u);
 	/** Returns prefix for an index (starting at zero). All prefixes can be traversed by starting at index zero and increasing the index until NULL is returned.
 	*
 	* @param index Index of prefix.
@@ -960,6 +964,7 @@ class Calculator {
 	void addMessages(vector<CalculatorMessage> *message_vector);
 	const PrintOptions &messagePrintOptions() const;
 	void setMessagePrintOptions(const PrintOptions &po);
+	void cleanMessages(const MathStructure &mstruct, size_t first_message = 1);
 	//@}
 
 	/** @name Functions for loading and saving definitions (variables, functions, units, etc.). */
@@ -1017,7 +1022,7 @@ class Calculator {
 	* @param is_user_defs true if the definitions are local, false if they are global.
 	* @returns true if the definitions were successfully loaded.
 	*/
-	int loadDefinitions(const char *file_name, bool is_user_defs = true);
+	int loadDefinitions(const char *file_name, bool is_user_defs = true, bool check_duplicates_of_global = false);
 	/** Save local definitions to ~/.qalculate/definitions/
 	*
 	* @returns true if definitions was successfully saved.
@@ -1049,7 +1054,7 @@ class Calculator {
 	bool hasGVFS();
 	///Deprecated: gvfs is not needed anymore.
 	bool hasGnomeVFS();
-	/** Load saved (local) currency units and exchange rates.
+	/** Load exchange rates. Use before loadGlobalCurrencies() or loadGlobalDefinitions().
 	*
 	* @returns true if operation successful.
 	*/
@@ -1141,6 +1146,11 @@ class Calculator {
 	void beginTemporaryEnableIntervalArithmetic();
 	void endTemporaryEnableIntervalArithmetic();
 	//@}
+	
+	void setCustomInputBase(Number nr);
+	void setCustomOutputBase(Number nr);
+	const Number &customInputBase() const;
+	const Number &customOutputBase() const;
 
 	/** @name Functions for localization */
 	//@{
@@ -1151,8 +1161,9 @@ class Calculator {
 	const string &getComma() const;
 	/** Sets argument separator and decimal sign from the current locale. Mainly for internal use. */
 	void setLocale();
-	/** If "C" locale shall be used. */
+	///Deprecated: use pass true to constructor instead
 	void setIgnoreLocale();
+	bool getIgnoreLocale();
 	void useDecimalComma();
 	/** Use point as decimal separator. 
 	* To use comma as an ignored separator in numbers, must be invoked with comma_as_separator = true, to change the default function argument separator to semicolon, in addition to using ParseOptions::comma_as_separator.
